import copy
import typing as t
from functools import partial

import casefy
import reflex as rx
from deepmerge import always_merger
from reflex.utils.imports import ImportDict

from RIL._core import Base
from RIL.settings import settings
from RIL import utils

__all__ = ["fontawesome", "fa"]


class FontAwesomeIcon(Base):
    library = "@fortawesome/react-fontawesome"
    tag = "FontAwesomeIcon"

    icon: rx.Var[str]
    """
    The name of the icon.
    """

    size: rx.Var[
        t.Literal[
            "2xs",
            "xs",
            "sm",
            "lg",
            "xl",
            "2xl",
            "1x",
            "2x",
            "3x",
            "4x",
            "5x",
            "6x",
            "7x",
            "8x",
            "9x",
            "10x",
        ]
    ]
    """
    The size of the icon.
    """

    fixed_width: rx.Var[bool]
    """
    Whether the icon should have fixed with.
    """

    list_item: rx.Var[bool]
    """
    Whether the icon should be displayed as a list item.
    """

    rotation: rx.Var[t.Literal[90, 180, 270]]
    """
    The icon's angle of rotation.
    """

    flip: rx.Var[t.Literal["horizontal", "vertical", "both"]]
    """
    How the icon should be flipped.
    """

    animation: rx.Var[str]
    """
    An animation to apply to the icon.

    For possible values, see: https://docs.fontawesome.com/web/use-with/react/style#animate-icons
    """

    border: rx.Var[bool]
    """
    Whether to add a border around the icon.
    """

    pull: rx.Var[t.Literal["left", "right"]]
    """
    Pull the icon to the left or right. Useful for wrapping text around an icon.
    """

    transform: rx.Var[str]
    """
    A space-separated list of transforms to apply to the icon. (e.g., "shrink-6 left-4 rotate-42").

    For possible values, see: https://docs.fontawesome.com/web/use-with/react/style#animate-icons
    """

    mask: rx.Var["FontAwesomeIcon"]
    """
    Another icon to mask this one with. This must be a `FontAwesomeIcon` component, and not simply the name of an icon.
    Styling options passed to the masking icon will have no effect.
    """

    swap_opacity: rx.Var[bool]
    """
    If this is a duotone icon, whether to swap the opacity of its layers. Has no effect on non-duotone icons.
    """

    inverse: rx.Var[bool]
    """
    Whether to invert the icon's colors.
    """

    def add_imports(self, **imports) -> ImportDict | list[ImportDict]:
        return imports

    @classmethod
    def _get_package_for_style(cls, style: str) -> str:
        # If a Kit exists, we use it.
        if settings.fontawesome.kit_package:
            # For custom icons, the module path is "kit/custom".
            if style == "kit":
                module_path = "kit/custom"
            else:
                # The module path is determined by replacing the last hyphen (-) in a style's name
                # with a forward slash (/) (e.g., "sharp-duotone-solid" becomes "sharp-duotone/solid"). We do this
                # by reversing the style name, replacing the *first* hyphen in the reversed string, and then
                # re-reversing the result.
                module_path = style[::-1].replace("-", "/", 1)[::-1]

            return f"{settings.fontawesome.kit_package}/icons/{module_path}"

        # At this point, we know we aren't using a Kit, so we need to figure out what package
        # we *are* using.

        # Since we're not using a Kit, custom icons should raise a `ValueError`.
        if style == "kit":
            raise ValueError(
                f"Additional configuration is required to use custom Font Awesome icons. "
                f"{utils.docs('fontawesome/pro/#using-a-kit')}"
            )

        # Brand icons always use @fortawesome/free-brands-svg-icons.
        if style == "classic-brands":
            return "@fortawesome/free-brands-svg-icons"

        # If Font Awesome Pro is enabled, we use the appropriate Pro package.
        if settings.fontawesome.pro_enabled:
            # Duotone Solid is currently a special case.
            if style == "duotone-solid":
                return "@fortawesome/pro-duotone-svg-icons"

            # Classic styles need to be prefixed with "pro-" to get the package name.
            style = style.replace("classic-", "pro-", 1)

            return f"@fortawesome/{style}-svg-icons"

        # If we still haven't returned, our only remaining options are the free Classic Solid and
        # Classic Regular packages.

        if style == "classic-solid":
            return "@fortawesome/free-solid-svg-icons"

        if style == "classic-regular":
            return "@fortawesome/free-regular-svg-icons"

        # Only thing left to do at this point is raise an exception.
        raise ValueError(
            f"The {' '.join(style.split('-')).title()} style requires Font Awesome Pro. {utils.docs('fontawesome/pro')}"
        )

    @classmethod
    def _normalize_icon_name(cls, icon_name: str) -> str:
        return "fa" + "".join(
            [i.capitalize() for i in icon_name.removeprefix("fa-").split("-")]
        )

    @classmethod
    def _get_icon_alias(cls, icon_name: str, icon_style: str) -> str:
        return (
            "fa"
            + casefy.pascalcase(icon_style)
            + cls._normalize_icon_name(icon_name).removeprefix("fa")
        )

    @classmethod
    def create(cls, icon: str = None, _icon_style: str = None, **props) -> t.Self:
        # The icon name is normalized to fa{Icon} and given an alias suffixed with a UUID to avoid
        # collisions with any sister icons in different styles.
        tag = cls._normalize_icon_name(icon)
        alias = cls._get_icon_alias(icon, _icon_style)

        # Determine the package this icon should be imported from.
        package = cls._get_package_for_style(_icon_style)

        # These props need to be overridden. Format: {name: (rx.Var[type], default_value)}
        props_to_override = {"icon": (rx.Var[t.Any], rx.Var(alias))}

        # The component will depend on @fortawesome/fontawesome-svg-core and the Kit package (if a Kit exists)
        # or the appropriate @fortawesome package (if no Kit exists).

        if settings.fontawesome.kit_package:
            second_dependency = f"{settings.fontawesome.kit_package}@latest"
        else:
            second_dependency = package

        lib_dependencies = [
            "@fortawesome/fontawesome-svg-core",
            second_dependency,
        ]

        # The component needs to import the icon from the appropriate package.
        imports = {package: [rx.ImportVar(tag, alias=alias, install=False)]}

        # The `animation` prop is applied as a boolean prop of the animation's name.
        animation = props.get("animation")
        if isinstance(animation, str):
            props.pop("animation")
            props_to_override.update(
                {animation: (bool, True) for animation in animation.split(" ")}
            )

        # If a mask was provided, we need to combine the component's dependencies and imports with those
        # of the mask.
        mask = props.get("mask")
        if isinstance(mask, cls):
            props.pop("mask")
            lib_dependencies.extend(mask.lib_dependencies)
            # noinspection PyArgumentList
            imports = always_merger.merge(
                copy.deepcopy(imports), mask.add_imports(mask)
            )

            # The actual value of the mask prop is the raw JSX reference to the masking icon.
            props_to_override["mask"] = (rx.Var[t.Any], mask.icon)
        elif mask is not None:
            raise TypeError(
                f"The mask of a Font Awesome icon must be another Font Awesome icon and not {type(mask)}"
            )

        # We create a new component class as a subclass of this one, overriding props as necessary.
        component_model = cls._reproduce(
            lib_dependencies=(list[str], lib_dependencies),
            **props_to_override,
        )

        # We override `add_imports` to import the icon.
        component_model.add_imports = partial(component_model.add_imports, **imports)

        # Finally, we return an instance of the new component class.
        return component_model._create(**props)


class FontAwesomeSharp(rx.ComponentNamespace):
    solid = __call__ = staticmethod(
        partial(staticmethod(FontAwesomeIcon.create), _icon_style="sharp-solid")
    )
    regular = partial(staticmethod(FontAwesomeIcon.create), _icon_style="sharp-regular")
    light = partial(staticmethod(FontAwesomeIcon.create), _icon_style="sharp-light")
    thin = partial(staticmethod(FontAwesomeIcon.create), _icon_style="sharp-thin")


class FontAwesomeDuotone(rx.ComponentNamespace):
    solid = __call__ = staticmethod(
        partial(staticmethod(FontAwesomeIcon.create), _icon_style="duotone-solid")
    )
    regular = partial(
        staticmethod(FontAwesomeIcon.create), _icon_style="duotone-regular"
    )
    light = partial(staticmethod(FontAwesomeIcon.create), _icon_style="duotone-light")
    thin = partial(staticmethod(FontAwesomeIcon.create), _icon_style="duotone-thin")


class FontAwesomeSharpDuotone(rx.ComponentNamespace):
    solid = __call__ = staticmethod(
        partial(staticmethod(FontAwesomeIcon.create), _icon_style="sharp-duotone-solid")
    )
    regular = partial(
        staticmethod(FontAwesomeIcon.create), _icon_style="sharp-duotone-regular"
    )
    light = partial(
        staticmethod(FontAwesomeIcon.create), _icon_style="sharp-duotone-light"
    )
    thin = partial(
        staticmethod(FontAwesomeIcon.create), _icon_style="sharp-duotone-thin"
    )


class FontAwesome(rx.ComponentNamespace):
    solid = __call__ = staticmethod(
        partial(FontAwesomeIcon.create, _icon_style="classic-solid")
    )
    regular = partial(
        staticmethod(FontAwesomeIcon.create), _icon_style="classic-regular"
    )
    light = partial(staticmethod(FontAwesomeIcon.create), _icon_style="classic-light")
    thin = partial(staticmethod(FontAwesomeIcon.create), _icon_style="classic-thin")
    brands = partial(staticmethod(FontAwesomeIcon.create), _icon_style="classic-brands")
    kit = partial(staticmethod(FontAwesomeIcon.create), _icon_style="kit")
    sharp = FontAwesomeSharp()
    duotone = FontAwesomeDuotone()
    sharp_duotone = FontAwesomeSharpDuotone()


fontawesome = fa = FontAwesome()



import typing as t

import reflex as rx

from RIL._core import Base

class FontAwesomeIcon(Base):
    @classmethod
    def create(
        cls,
        icon: str,
        size: t.Literal[
            "2xs",
            "xs",
            "sm",
            "lg",
            "xl",
            "2xl",
            "1x",
            "2x",
            "3x",
            "4x",
            "5x",
            "6x",
            "7x",
            "8x",
            "9x",
            "10x",
        ] = None,
        fixed_width: bool = None,
        list_item: bool = None,
        rotation: t.Literal[90, 180, 270] = None,
        flip: t.Literal["horizontal", "vertical", "both"] = None,
        animation: str = None,
        border: bool = None,
        pull: t.Literal["left", "right"] = None,
        transform: str = None,
        mask: FontAwesomeIcon = None,
        swap_opacity: bool = None,
        inverse: bool = None,
    ) -> None:
        """
        Create a FontAwesome icon.

        Parameters
        ----------
        icon : str
            The name of the icon.

        size : {"2xs", "xs", "sm", "lg", "xl", "2xl", "1x", "2x", "3x", "4x", "5x", "6x", "7x", "8x", "9x", "10x"}, optional
            The size of the icon.

        fixed_width : bool, optional
            Whether the icon should have a fixed width.

        list_item : bool, optional
            Whether the icon should be displayed as a list item.

        rotation : {90, 180, 270}, optional
            The icon's angle of rotation.

        flip : {"horizontal", "vertical", "both"}, optional
            How the icon should be flipped.

        animation : str, optional
            An animation to apply to the icon.
            For possible values, see: https://docs.fontawesome.com/web/use-with/react/style#animate-icons

        border : bool, optional
            Whether to add a border around the icon.

        pull : {"left", "right"}, optional
            Pull the icon to the left or right. Useful for wrapping text around an icon.

        transform : str, optional
            A space-separated list of transforms to apply to the icon (e.g., `"shrink-6 left-4 rotate-42"`).
            For possible values, see: https://docs.fontawesome.com/web/use-with/react/style#animate-icons

        mask : FontAwesomeIcon, optional
            Another icon to mask this one with. This must be a `FontAwesomeIcon` component, not simply the name of an icon.
            Styling options passed to the masking icon will have no effect.

        swap_opacity : bool, optional
            If this is a duotone icon, whether to swap the opacity of its layers. Has no effect on non-duotone icons.

        inverse : bool, optional
            Whether to invert the icon's colors.
        """

class FontAwesomeSharp(rx.ComponentNamespace):
    solid = regular = light = thin = FontAwesomeIcon.create

    @staticmethod
    def __call__(
        icon: str,
        size: t.Literal[
            "2xs",
            "xs",
            "sm",
            "lg",
            "xl",
            "2xl",
            "1x",
            "2x",
            "3x",
            "4x",
            "5x",
            "6x",
            "7x",
            "8x",
            "9x",
            "10x",
        ] = None,
        fixed_width: bool = None,
        list_item: bool = None,
        rotation: t.Literal[90, 180, 270] = None,
        flip: t.Literal["horizontal", "vertical", "both"] = None,
        animation: str = None,
        border: bool = None,
        pull: t.Literal["left", "right"] = None,
        transform: str = None,
        mask: FontAwesomeIcon = None,
        swap_opacity: bool = None,
        inverse: bool = None,
    ) -> None:
        """
        Create a FontAwesome icon.

        Parameters
        ----------
        icon : str
            The name of the icon.

        size : {"2xs", "xs", "sm", "lg", "xl", "2xl", "1x", "2x", "3x", "4x", "5x", "6x", "7x", "8x", "9x", "10x"}, optional
            The size of the icon.

        fixed_width : bool, optional
            Whether the icon should have a fixed width.

        list_item : bool, optional
            Whether the icon should be displayed as a list item.

        rotation : {90, 180, 270}, optional
            The icon's angle of rotation.

        flip : {"horizontal", "vertical", "both"}, optional
            How the icon should be flipped.

        animation : str, optional
            An animation to apply to the icon.
            For possible values, see: https://docs.fontawesome.com/web/use-with/react/style#animate-icons

        border : bool, optional
            Whether to add a border around the icon.

        pull : {"left", "right"}, optional
            Pull the icon to the left or right. Useful for wrapping text around an icon.

        transform : str, optional
            A space-separated list of transforms to apply to the icon (e.g., `"shrink-6 left-4 rotate-42"`).
            For possible values, see: https://docs.fontawesome.com/web/use-with/react/style#animate-icons

        mask : FontAwesomeIcon, optional
            Another icon to mask this one with. This must be a `FontAwesomeIcon` component, not simply the name of an icon.
            Styling options passed to the masking icon will have no effect.

        swap_opacity : bool, optional
            If this is a duotone icon, whether to swap the opacity of its layers. Has no effect on non-duotone icons.

        inverse : bool, optional
            Whether to invert the icon's colors.
        """

class FontAwesomeDuotone(rx.ComponentNamespace):
    solid = regular = light = thin = FontAwesomeIcon.create

    @staticmethod
    def __call__(
        icon: str,
        size: t.Literal[
            "2xs",
            "xs",
            "sm",
            "lg",
            "xl",
            "2xl",
            "1x",
            "2x",
            "3x",
            "4x",
            "5x",
            "6x",
            "7x",
            "8x",
            "9x",
            "10x",
        ] = None,
        fixed_width: bool = None,
        list_item: bool = None,
        rotation: t.Literal[90, 180, 270] = None,
        flip: t.Literal["horizontal", "vertical", "both"] = None,
        animation: str = None,
        border: bool = None,
        pull: t.Literal["left", "right"] = None,
        transform: str = None,
        mask: FontAwesomeIcon = None,
        swap_opacity: bool = None,
        inverse: bool = None,
    ) -> None:
        """
        Create a FontAwesome icon.

        Parameters
        ----------
        icon : str
            The name of the icon.

        size : {"2xs", "xs", "sm", "lg", "xl", "2xl", "1x", "2x", "3x", "4x", "5x", "6x", "7x", "8x", "9x", "10x"}, optional
            The size of the icon.

        fixed_width : bool, optional
            Whether the icon should have a fixed width.

        list_item : bool, optional
            Whether the icon should be displayed as a list item.

        rotation : {90, 180, 270}, optional
            The icon's angle of rotation.

        flip : {"horizontal", "vertical", "both"}, optional
            How the icon should be flipped.

        animation : str, optional
            An animation to apply to the icon.
            For possible values, see: https://docs.fontawesome.com/web/use-with/react/style#animate-icons

        border : bool, optional
            Whether to add a border around the icon.

        pull : {"left", "right"}, optional
            Pull the icon to the left or right. Useful for wrapping text around an icon.

        transform : str, optional
            A space-separated list of transforms to apply to the icon (e.g., `"shrink-6 left-4 rotate-42"`).
            For possible values, see: https://docs.fontawesome.com/web/use-with/react/style#animate-icons

        mask : FontAwesomeIcon, optional
            Another icon to mask this one with. This must be a `FontAwesomeIcon` component, not simply the name of an icon.
            Styling options passed to the masking icon will have no effect.

        swap_opacity : bool, optional
            If this is a duotone icon, whether to swap the opacity of its layers. Has no effect on non-duotone icons.

        inverse : bool, optional
            Whether to invert the icon's colors.
        """

class FontAwesomeSharpDuotone(rx.ComponentNamespace):
    solid = regular = light = thin = FontAwesomeIcon.create

    @staticmethod
    def __call__(
        icon: str,
        size: t.Literal[
            "2xs",
            "xs",
            "sm",
            "lg",
            "xl",
            "2xl",
            "1x",
            "2x",
            "3x",
            "4x",
            "5x",
            "6x",
            "7x",
            "8x",
            "9x",
            "10x",
        ] = None,
        fixed_width: bool = None,
        list_item: bool = None,
        rotation: t.Literal[90, 180, 270] = None,
        flip: t.Literal["horizontal", "vertical", "both"] = None,
        animation: str = None,
        border: bool = None,
        pull: t.Literal["left", "right"] = None,
        transform: str = None,
        mask: FontAwesomeIcon = None,
        swap_opacity: bool = None,
        inverse: bool = None,
    ) -> None:
        """
        Create a FontAwesome icon.

        Parameters
        ----------
        icon : str
            The name of the icon.

        size : {"2xs", "xs", "sm", "lg", "xl", "2xl", "1x", "2x", "3x", "4x", "5x", "6x", "7x", "8x", "9x", "10x"}, optional
            The size of the icon.

        fixed_width : bool, optional
            Whether the icon should have a fixed width.

        list_item : bool, optional
            Whether the icon should be displayed as a list item.

        rotation : {90, 180, 270}, optional
            The icon's angle of rotation.

        flip : {"horizontal", "vertical", "both"}, optional
            How the icon should be flipped.

        animation : str, optional
            An animation to apply to the icon.
            For possible values, see: https://docs.fontawesome.com/web/use-with/react/style#animate-icons

        border : bool, optional
            Whether to add a border around the icon.

        pull : {"left", "right"}, optional
            Pull the icon to the left or right. Useful for wrapping text around an icon.

        transform : str, optional
            A space-separated list of transforms to apply to the icon (e.g., `"shrink-6 left-4 rotate-42"`).
            For possible values, see: https://docs.fontawesome.com/web/use-with/react/style#animate-icons

        mask : FontAwesomeIcon, optional
            Another icon to mask this one with. This must be a `FontAwesomeIcon` component, not simply the name of an icon.
            Styling options passed to the masking icon will have no effect.

        swap_opacity : bool, optional
            If this is a duotone icon, whether to swap the opacity of its layers. Has no effect on non-duotone icons.

        inverse : bool, optional
            Whether to invert the icon's colors.
        """

class FontAwesome(rx.ComponentNamespace):
    solid = regular = light = thin = brands = kit = FontAwesomeIcon.create
    sharp = FontAwesomeSharp()
    duotone = FontAwesomeDuotone()
    sharp_duotone = FontAwesomeSharpDuotone()

    @staticmethod
    def __call__(
        icon: str,
        size: t.Literal[
            "2xs",
            "xs",
            "sm",
            "lg",
            "xl",
            "2xl",
            "1x",
            "2x",
            "3x",
            "4x",
            "5x",
            "6x",
            "7x",
            "8x",
            "9x",
            "10x",
        ] = None,
        fixed_width: bool = None,
        list_item: bool = None,
        rotation: t.Literal[90, 180, 270] = None,
        flip: t.Literal["horizontal", "vertical", "both"] = None,
        animation: str = None,
        border: bool = None,
        pull: t.Literal["left", "right"] = None,
        transform: str = None,
        mask: FontAwesomeIcon = None,
        swap_opacity: bool = None,
        inverse: bool = None,
    ) -> None:
        """
        Create a FontAwesome icon.

        Parameters
        ----------
        icon : str
            The name of the icon.

        size : {"2xs", "xs", "sm", "lg", "xl", "2xl", "1x", "2x", "3x", "4x", "5x", "6x", "7x", "8x", "9x", "10x"}, optional
            The size of the icon.

        fixed_width : bool, optional
            Whether the icon should have a fixed width.

        list_item : bool, optional
            Whether the icon should be displayed as a list item.

        rotation : {90, 180, 270}, optional
            The icon's angle of rotation.

        flip : {"horizontal", "vertical", "both"}, optional
            How the icon should be flipped.

        animation : str, optional
            An animation to apply to the icon.
            For possible values, see: https://docs.fontawesome.com/web/use-with/react/style#animate-icons

        border : bool, optional
            Whether to add a border around the icon.

        pull : {"left", "right"}, optional
            Pull the icon to the left or right. Useful for wrapping text around an icon.

        transform : str, optional
            A space-separated list of transforms to apply to the icon (e.g., `"shrink-6 left-4 rotate-42"`).
            For possible values, see: https://docs.fontawesome.com/web/use-with/react/style#animate-icons

        mask : FontAwesomeIcon, optional
            Another icon to mask this one with. This must be a `FontAwesomeIcon` component, not simply the name of an icon.
            Styling options passed to the masking icon will have no effect.

        swap_opacity : bool, optional
            If this is a duotone icon, whether to swap the opacity of its layers. Has no effect on non-duotone icons.

        inverse : bool, optional
            Whether to invert the icon's colors.
        """

fontawesome = fa = FontAwesome()



import typing as t
from functools import partial

from pydantic import field_serializer, Field, BaseModel
from pydantic_extra_types.color import Color
from reflex.utils.imports import ImportDict

from RIL._core import Base, Props, validate_props
from RIL.settings import settings
import reflex as rx
from loguru import logger
import hishel
import httpx
import semver

__all__ = ["simple", "si"]


class SimpleIconsPackage(BaseModel):
    base_package: t.ClassVar[str] = "@icons-pack/react-simple-icons"

    name: str

    @property
    def component_library(self) -> str:
        return self.name.split("@npm:")[0]

    @property
    def import_dict(self) -> dict:
        return {self.name: rx.ImportVar(None, render=False)}

    @classmethod
    def at(cls, version: semver.Version | t.Literal["latest"]) -> t.Self:
        if str(version) == "latest":
            return cls(name=cls.base_package)

        return cls(
            name=f"{cls.base_package}-{version}@npm:{cls.base_package}@{version}"
        )


class SimpleIconProps(Props):
    title: str = None
    """
    A short, accessible, description of the icon.
    """

    color: Color | t.Literal["default"] = None
    """
    The color of this icon. May be:
    - a hex code (e.g., `"#03cb98"`)
    - an tuple of RGB, RBGA, or HSL values
    - `"default"`, which makes the icon use whatever color Simple Icons has chosen as its default
    - any valid color name as determined by the CSS Color Module Level 3 specification 
    (https://www.w3.org/TR/css-color-3/#svg-color)
    
    Hex codes are case-insensitive and the leading `#` is optional.
    """

    size: int | str = None
    """
    The size of the icon. May be an integer (in pixels) or a CSS size string (e.g., `'1rem'`).
    """

    version: int | t.Literal["latest"] = Field(
        settings.simple.version, ge=10, exclude=True
    )
    """
    The major version of Simple Icons to use for this icon. May be "latest" or an integer 
    greater than or equal to 10.
    
    Defaults to the value of the `simple.version` setting.
    """

    @property
    def package(self) -> SimpleIconsPackage:
        if self.version == "latest":
            return SimpleIconsPackage.at("latest")

        with hishel.CacheClient(base_url="https://registry.npmjs.org") as npm:
            try:
                resp = npm.get(SimpleIconsPackage.base_package)
                resp.raise_for_status()
            except httpx.HTTPError:
                logger.critical(
                    f"RIL could not determine the version of {SimpleIconsPackage.base_package} to install "
                    "because it could not access the NPM registry. You can suppress this error "
                    'by setting the simple.version setting to "latest".'
                )

                exit(1)

            package_info = resp.json()

            for release in reversed(package_info["versions"].values()):
                package_version = semver.Version.parse(release["version"])

                if package_version.prerelease:
                    continue

                package_si_version = semver.Version.parse(
                    release["devDependencies"]["simple-icons"]
                )

                if package_si_version.major <= self.version:
                    return SimpleIconsPackage.at(package_version)
            else:
                logger.critical(
                    f"No version of {SimpleIconsPackage.base_package} could be sound for "
                    f"Simple Icons <= {self.version}."
                )

                exit(1)

    @field_serializer("color")
    def serialize_color_as_hex(self, color: Color | t.Literal["default"] | None):
        return color.as_hex() if color and color != "default" else color


class SimpleIcon(Base):
    @property
    def import_var(self):
        return rx.ImportVar(self.tag, install=False)

    def add_imports(self, **imports) -> ImportDict | list[ImportDict]:
        return imports

    @classmethod
    @validate_props
    def create(cls, icon: str, props: SimpleIconProps):
        component_model = cls._reproduce(props=props.model_dump())
        component_model.add_imports = partial(
            component_model.add_imports, **props.package.import_dict
        )

        component = component_model._create(**props.model_dump())
        component.library = props.package.component_library
        component.tag = "Si" + icon.replace(" ", "").replace(".", "dot").capitalize()

        return component


simple = si = SimpleIcon.create



import copy
import sys
import typing as t

import pydantic
import pydantic.v1
import reflex as rx
from loguru import logger
from pydantic import BaseModel, ConfigDict, model_serializer, validate_call


class Props(BaseModel):
    model_config = ConfigDict(extra="allow")

    def model_dump(self, **kwargs):
        return super().model_dump(**kwargs, exclude_none=True, by_alias=True)

    @model_serializer(mode="wrap")
    def serialize(self, handler: t.Callable):
        serialized = handler(self)
        reserialized = copy.deepcopy(serialized)

        if self.__pydantic_extra__:
            for key, value in serialized.items():
                if key in self.__pydantic_extra__:
                    reserialized.pop(key)
                    reserialized.setdefault("_extra", {})[key] = value

        return reserialized


class Base(rx.Component):
    """
    Base class for all components in this library.
    """

    @classmethod
    @validate_call
    def _reproduce(cls, *, props: dict = None, **fields):
        props = props or {}

        # Futureproofing for Reflex's planned move to Pydantic v2. https://github.com/reflex-dev/reflex/issues/1539
        if issubclass(cls, pydantic.v1.BaseModel):
            create_model = pydantic.v1.create_model
        else:
            create_model = pydantic.create_model

        if isinstance(props, Props):
            props = props.model_dump()

        model = create_model(
            cls.__name__,
            __base__=cls,
            **{k: (rx.Var[t.Any], v) for k, v in props.items()},
            **fields,
        )

        return model

    @classmethod
    def _create(cls, *children, **props) -> rx.Component:
        return super().create(*children, **props)


def validate_props(func):
    def wrapper(*args, **props):
        return validate_call(func)(*args, props=props)

    return wrapper


log_level = rx.config.get_config().loglevel

if log_level.casefold() == "default":
    log_level = "warning"

logger.remove()
logger.add(sink=sys.stderr, level=log_level.upper(), colorize=True)



"""
A Reflex component providing access to GitHub's Octicons.

https://ril.celsiusnarhwal.dev/octicons
"""

import typing as t

from RIL._core import Base

__all__ = ["octicons"]

class Octicon(Base):
    @classmethod
    def create(
        cls,
        icon: str,
        color: str | tuple = None,
        size: int | str | t.Literal["small", "medium", "large"] = None,
    ):
        """
        Create an Octicon.

        Parameters
        ----------
        icon : str
            The name of the icon.

        color : str | tuple, optional
            The color of the icon. May be:
            - a hex code (e.g., `"#03cb98"`)
            - a integer tuple of RGB values, with an optional fourth value for transparency (e.g., `(3, 203, 152, 1)`)
            - any valid color name as determined by the CSS Color Module Level 3 specification
            (https://www.w3.org/TR/css-color-3/#svg-color)

            Hex codes are case-insensitive and the leading `#` is optional.

        size : int | str | {"small", "medium", "large"}, optional
            The size of the icon. May be `"small"`, `"medium"`, or `"large"`. May also be an integer (in pixels) or
            a CSS size string (e.g., `'1rem'`).
        """

octicons = Octicon.create



import re
import typing as t

import casefy
import inflect as ifl
import reflex as rx
from pydantic import Field, field_serializer, model_serializer
from pydantic_extra_types.color import Color
from reflex.utils.imports import ImportDict

from RIL._core import Base, Props, validate_props

__all__ = ["material"]

inflect = ifl.engine()


class MaterialSymbolProps(Props):
    variant: t.Literal["outlined", "rounded", "sharp"] = Field("outlined", exclude=True)
    """
    The variant of the icon. May be either `"outlined"`, `"rounded"`, or `"sharp"`. Defaults to `"outlined"`.
    """

    filled: bool = Field(False, exclude=True)
    """
    Whether or not to use the icon's filled appearance.
    """

    color: Color = Field(None, serialization_alias="fill")
    """
    The color of the icon. May be:
    - a hex code (e.g., `"#03cb98"`)
    - a integer tuple of RGB values, with an optional fourth value for transparency (e.g., `(3, 203, 152, 1)`)
    - any valid color name as determined by the CSS Color Module Level 3 specification 
    (https://www.w3.org/TR/css-color-3/#svg-color)
    
    Hex codes are case-insensitive and the leading `#` is optional.
    """

    size: int | str = Field(None, exclude=True)
    """
    The size of the icon. May be an integer (in pixels) or a CSS size string (e.g., `'1rem'`).
    
    See Also
        https://developer.mozilla.org/en-US/docs/Web/CSS/length
    """

    @field_serializer("color")
    def serialize_color_as_hex(self, color: Color | None):
        return color.as_hex() if color else color

    @model_serializer(mode="wrap")
    def serialize(self, handler: t.Callable):
        serialized = super().serialize(handler)

        if self.size:
            serialized["height"] = serialized["width"] = self.size

        return serialized


class MaterialSymbol(Base):
    @property
    def import_var(self):
        return rx.ImportVar(tag=self.tag, alias=self.alias, install=False)

    def add_imports(self) -> ImportDict | list[ImportDict]:
        return {
            "@nine-thirty-five/material-symbols-react@^1": rx.ImportVar(
                None, render=False, transpile=True
            )
        }

    @classmethod
    @validate_props
    def create(cls, icon: str, props: MaterialSymbolProps) -> rx.Component:
        component_model = cls._reproduce(props=props.model_dump())

        component = component_model._create(**props.model_dump())
        component.library = f"@nine-thirty-five/material-symbols-react/{props.variant}"

        if props.filled:
            component.library += "/filled"

        # The tag is the supplied icon name with whitespace removed and any numbers converted to their PascalCase
        # word forms. Consecutive numbers are treated as a group — e.g., "E911 Avatar" becomes
        # "ENineHundredElevenAvatar" and not "ENineOneOneAvatar".

        component.tag = re.sub(
            r"\d+",
            lambda m: casefy.pascalcase(inflect.number_to_words(m.group(), andword="")),
            icon,
        )

        component.tag = component.tag.replace(" ", "")

        # The alias is "Material" + the tag + the variant + "Filled" if the icon is to use its
        # filled appearance.

        component.alias = (
            "Material"
            + component.tag
            + casefy.pascalcase(" ".join(component.library.split("/")[2:]))
        )

        return component


material = MaterialSymbol.create



from RIL._core import Base
import typing as t

class SimpleIcon(Base):
    @classmethod
    def create(
        cls,
        icon: str,
        title: str = None,
        color: str | tuple = None,
        size: str | int = None,
        version: int | t.Literal["default"] = None,
    ):
        """
        Create a Simple Icon.

        Parameters
        ----------
        icon : str
            The name of the icon.

        title : str, optional
         A short, accessible, description of the icon.

        color : str | tuple, optional
            The color of this icon. May be:
            - a hex code
            - a tuple of RGB, RGBA or HSL values
            - `"default"`, which makes the icon use whatever color Simple Icons has chosen as its default
            - any valid color name as determined by the CSS Color Module Level 3 specification

            Hex codes are case-insensitive and the leading `#` is optional..

        size : str | int, optional
            The size of the icon. May be an integer (in pixels) or a CSS size string (e.g., `'1rem'`).

        version : int | {"default"}, optional
            The major version of Simple Icons to use for this icon. May be "latest" or an integer
            greater than or equal to 10.

            Defaults to the value of the `simple.version` setting.
        """

simple = si = SimpleIcon.create



import os
import typing as t
from pathlib import Path

from jinja2 import Environment, FileSystemLoader
from loguru import logger
from pydantic import AnyHttpUrl, BaseModel, Field, model_validator
from pydantic_extra_types.color import Color
from pydantic_settings import (
    BaseSettings,
    PydanticBaseSettingsSource,
    PyprojectTomlConfigSettingsSource,
    SettingsConfigDict,
    TomlConfigSettingsSource,
)
from reflex.utils.prerequisites import get_web_dir

jinja = Environment(loader=FileSystemLoader(Path(__file__).parent / "templates"))


class FontAwesomeSettings(BaseModel):
    """
    Settings for RIL's Font Awesome component.
    """

    pro_enabled: bool = False
    kit_code: str = None
    npm_registry: AnyHttpUrl = "https://npm.fontawesome.com"

    @property
    def kit_package(self) -> str | None:
        """
        Return the name of the Kit package if Font Awesome Pro is enabled and a Kit code is provided,
        or `None` otherwise.
        """
        if self.pro_enabled and self.kit_code:
            logger.debug(f"Using Font Awesome Kit {self.kit_code}")
            return f"@awesome.me/kit-{self.kit_code}"

    @model_validator(mode="after")
    def write_npmrc(self):
        """
        Write a .npmrc file to the Reflex web directory if Font Awesome Pro is enabled, or remove that file if
        it isn't.
        """
        userland_npmrc = get_web_dir() / ".npmrc"

        if not self.pro_enabled:
            userland_npmrc.unlink(missing_ok=True)
        else:
            npmrc_template = jinja.get_template(".npmrc.jinja")
            reference_npmrc = npmrc_template.render(registry=self.npm_registry)

            if userland_npmrc.parent.exists():
                userland_npmrc.write_text(reference_npmrc)

        return self


class SimpleIconsSettings(BaseModel):
    """
    Settings for RIL's Simple Icons component.
    """

    version: int | t.Literal["latest"] = Field("latest", ge=10)


class PhosphorSettings(BaseModel):
    """
    Settings for RIL's Phosphor Icons component.
    """

    color: Color = None
    size: int | str = None
    weight: t.Literal["thin", "light", "regular", "bold", "fill", "duotone"] = None

    @property
    def provider_settings(self) -> dict:
        return self.model_dump(exclude_none=True)


class FontAwesomePackageTokenSetting(BaseSettings):
    """
    A special settings class that exists solely to consume the `FONTAWESOME_PACKAGE_TOKEN` environment variable
    via the current process or a `.env` file.
    """

    model_config = SettingsConfigDict(env_file=".env", extra="ignore")

    fontawesome_package_token: str = ""

    @classmethod
    def settings_customise_sources(
        cls,
        settings_cls: type[BaseSettings],
        init_settings: PydanticBaseSettingsSource,
        env_settings: PydanticBaseSettingsSource,
        dotenv_settings: PydanticBaseSettingsSource,
        file_secret_settings: PydanticBaseSettingsSource,
    ) -> tuple[PydanticBaseSettingsSource, ...]:
        return (dotenv_settings,)

    @model_validator(mode="after")
    def set_token(self):
        os.environ.setdefault(
            "FONTAWESOME_PACKAGE_TOKEN", self.fontawesome_package_token
        )
        return self


class RILSettings(BaseSettings):
    model_config = SettingsConfigDict(
        env_prefix="RIL_",
        env_file=".env",
        env_nested_delimiter="__",
        pyproject_toml_table_header=("tool", "ril"),
        toml_file="ril.toml",
        extra="ignore",
    )

    fontawesome: FontAwesomeSettings = Field(default_factory=FontAwesomeSettings)
    simple: SimpleIconsSettings = Field(default_factory=SimpleIconsSettings)
    phosphor: PhosphorSettings = Field(default_factory=PhosphorSettings)

    @classmethod
    def settings_customise_sources(
        cls,
        settings_cls: type[BaseSettings],
        init_settings: PydanticBaseSettingsSource,
        env_settings: PydanticBaseSettingsSource,
        dotenv_settings: PydanticBaseSettingsSource,
        file_secret_settings: PydanticBaseSettingsSource,
    ) -> tuple[PydanticBaseSettingsSource, ...]:
        return (
            dotenv_settings,
            PyprojectTomlConfigSettingsSource(settings_cls),
            TomlConfigSettingsSource(settings_cls),
        )


settings = RILSettings()

# This just needs to be initialized.
FontAwesomePackageTokenSetting()



import re
import typing as t

import casefy
from pydantic import Field, field_serializer
from pydantic_extra_types.color import Color

from RIL._core import Base, Props, validate_props

__all__ = ["octicons"]


class OcticonProps(Props):
    color: Color = Field(None, serialization_alias="fill")
    """
    The color of the icon. May be:
    - a hex code
    - a tuple of RGB, RGBA, or HSL values
    - any valid color name as determined by the CSS Color Module Level 3 specification 
    (https://www.w3.org/TR/css-color-3/#svg-color)
    
    Hex codes are case-insensitive and the leading `#` is optional.
    """

    size: int | str | t.Literal["small", "medium", "large"] = None
    """
    The size of the icon. May be `"small"`, `"medium"`, or `"large"`. May also be an integer (in pixels) or 
    a CSS size string (e.g., `'1rem'`),
    """

    @field_serializer("color")
    def serialize_color_as_hex(self, color: Color | None):
        return color.as_hex() if color else color


class Octicon(Base):
    library = "@primer/octicons-react@^19"

    @classmethod
    @validate_props
    def create(cls, icon: str, props: OcticonProps):
        component_model = cls._reproduce(props=props.model_dump())
        component = component_model._create(**props.model_dump())

        icon = re.sub(r"icon$", "", icon, flags=re.I)
        component.tag = casefy.pascalcase(icon.casefold()) + "Icon"
        component.alias = "Octicons" + component.tag

        return component


octicons = Octicon.create



from RIL._core import Base

class BootstrapIcon(Base):
    @classmethod
    def create(
        cls,
        icon: str,
        color: str | tuple = None,
        size: int | str = None,
        title: str = None,
    ):
        """
        Create a Bootstrap Icon.

        Parameters
        ----------
        icon : str
            The name of the icon.

        color : str | tuple, optional
            The color of the icon. May be:
            - a hex code (e.g., `"#03cb98"`)
            - a integer tuple of RGB values, with an optional fourth value for transparency (e.g., `(3, 203, 152, 1)`)
            - any valid color name as determined by the CSS Color Module Level 3 specification
            (https://www.w3.org/TR/css-color-3/#svg-color)

            Hex codes are case-insensitive and the leading `#` is optional.

        size : int | str, optional
            The size of the icon. May be an integer (in pixels) or a CSS size string (e.g., `'1rem'`).

        title : str, optional
            An accessible, short-text, description of the icon.
        """

bootstrap = bi = BootstrapIcon.create



import typing as t

import casefy
import reflex as rx
from pydantic import ConfigDict, field_serializer, model_serializer
from pydantic_extra_types.color import Color
from reflex import Component
from reflex.utils.imports import ImportDict

from RIL._core import Base, Props, validate_props
from RIL.settings import settings

__all__ = ["phosphor"]


NPM_PACKAGE = "@phosphor-icons/react@^2"


class PhosphorIconProps(Props):
    weight: t.Literal["thin", "light", "regular", "bold", "fill", "duotone"] = None
    """
    The icon's weight (i.e., style). May be one of `"thin"`, `"light"`, `"regular"`, `"bold"`, `"fill"`, or `"duotone"`.
    """

    color: Color = None
    """
    The color of the icon. May be:
    - a hex code
    - a tuple of RGB, RGBA, or HSL values
    - any valid color name as determined by the CSS Color Module Level 3 specification 
    (https://www.w3.org/TR/css-color-3/#svg-color)
    
    Hex codes are case-insensitive and the leading `#` is optional.
    """

    size: int | str = None
    """
    The size of the icon. May be an integer (in pixels) or a CSS size string (e.g., `'1rem'`).
    
    See Also
        https://developer.mozilla.org/en-US/docs/Web/CSS/length
    """

    alt: str = None
    """
    Alt text for the icon.
    """

    @field_serializer("color")
    def serialize_color_as_hex(self, color: Color | None):
        return color.as_hex() if color else color


class PhosphorIconContextProps(PhosphorIconProps):
    model_config = ConfigDict(extra="ignore")

    # If we don't set a default here then context-wrapped icons will be *huge*.
    size: int | str = "1em"

    @model_serializer(mode="wrap")
    def serialize(self, handler: t.Callable):
        serialized = super().serialize(handler)
        return {"value": serialized}


class PhosphorIconContext(Base):
    tag = "PhosphorIconContext.Provider"

    def add_imports(self) -> ImportDict | list[ImportDict]:
        return {NPM_PACKAGE: [rx.ImportVar("IconContext", alias="PhosphorIconContext")]}

    @classmethod
    @validate_props
    def create(cls, *children, props: PhosphorIconContextProps) -> rx.Component:
        component_model = cls._reproduce(props=props.model_dump())
        return component_model._create(**props.model_dump())


class PhosphorIcon(Base):
    library = NPM_PACKAGE

    @classmethod
    @validate_props
    def create(cls, icon: str, props: PhosphorIconProps) -> rx.Component:
        component_model = cls._reproduce(props=props.model_dump())

        component = component_model._create(**props.model_dump())
        component.tag = casefy.pascalcase(icon.casefold())
        component.alias = "Phosphor" + component.tag

        return component

    @staticmethod
    def _get_app_wrap_components() -> dict[tuple[int, str], Component]:
        if settings.phosphor.provider_settings:
            return {
                (1, "PhosphorIconContext.Provider"): PhosphorIconContext.create(
                    **settings.phosphor.provider_settings
                )
            }

        return {}


class Phosphor(rx.ComponentNamespace):
    __call__ = staticmethod(PhosphorIcon.create)
    context = staticmethod(PhosphorIconContext.create)


phosphor = Phosphor()



import typing as t

from RIL._core import Base

class MaterialSymbol(Base):
    @classmethod
    def create(
        cls,
        icon: str,
        variant: t.Literal["outlined", "rounded", "sharp"] = None,
        filled: bool = None,
        color: str | tuple = None,
        size: int | str = None,
    ):
        """
        Create a Material Symbol.

        Parameters
        ----------
        icon : str
            The name of the icon.

        variant : {"outlined", "rounded", "sharp"}, optional
            The variant of the icon.

        filled : bool, optional
            Whether or not to use the icon's filled appearance.

        color : str | tuple, optional
            The color of the icon. May be:
            - a hex code (e.g., `"#03cb98"`)
            - a integer tuple of RGB values, with an optional fourth value for transparency (e.g., `(3, 203, 152, 1)`)
            - any valid color name as determined by the CSS Color Module Level 3 specification
            (https://www.w3.org/TR/css-color-3/#svg-color)

            Hex codes are case-insensitive and the leading `#` is optional.

        size : int | str, optional
            The size of the icon. May be an integer (in pixels) or a CSS size string (e.g., `'1rem'`).

        Returns
        -------

        """

material = MaterialSymbol.create



import typing as t

import reflex as rx

from RIL._core import Base

class PhosphorIcon(Base):
    @classmethod
    def create(
        cls,
        icon: str,
        weight: t.Literal["thin", "light", "regular", "bold", "fill", "duotone"] = None,
        color: str | tuple = None,
        size: int | str = None,
        alt: str = None,
    ) -> rx.Component:
        """
        Create a Phosphor icon.

        Parameters
        ----------
        icon : str
            The name of the icon.

        weight : typing.Literal["thin", "light", "regular", "bold", "fill", "duotone"], optional
            The icon's weight (i.e., style).

        color : str | tuple, optional
            The color of the icon. May be:
            - a hex code
            - a tuple of RGB, RGBA, or HSL values
            - any valid color name as determined by the CSS Color Module Level 3 specification
            (https://www.w3.org/TR/css-color-3/#svg-color)

            Hex codes are case-insensitive and the leading `#` is optional.

        size : int | str, optional
            The size of the icon. May be an integer (in pixels) or a CSS size string (e.g., `'1rem'`).

        alt : str, optional
            Alt text for the icon.
        """

class PhosphorIconContext(Base):
    @classmethod
    def create(
        cls,
        weight: t.Literal["thin", "light", "regular", "bold", "fill", "duotone"] = None,
        color: str | tuple = None,
        size: int | str = None,
    ) -> rx.Component:
        """
        Create a container that applies a set of default styling properties to all Phosphor icons within it.

        Parameters
        ----------
        weight : {"thin", "light", "regular", "bold", "fill", "duotone"}, optional
            The weight of the icons. (i.e., style).

        color : str | tuple, optional
            The color of the icons. May be:
            - a hex code (e.g., `"#03cb98"`)
            - a integer tuple of RGB values, with an optional fourth value for transparency (e.g., `(3, 203, 152, 1)`)
            - any valid color name as determined by the CSS Color Module Level 3 specification
            (https://www.w3.org/TR/css-color-3/#svg-color)

            Hex codes are case-insensitive and the leading `#` is optional.

        size : int | str, optional
            The size of the icons. May be an integer (in pixels) or a CSS size string (e.g., `'1rem'`).
        """

class Phosphor(rx.ComponentNamespace):
    context = staticmethod(PhosphorIconContext.create)

    @staticmethod
    def __call__(
        icon: str,
        weight: t.Literal["thin", "light", "regular", "bold", "fill", "duotone"] = None,
        color: str | tuple = None,
        size: int | str = None,
        alt: str = None,
    ) -> rx.Component:
        """
        Create a Phosphor icon.

        Parameters
        ----------
        icon : str
            The name of the icon.

        weight : typing.Literal["thin", "light", "regular", "bold", "fill", "duotone"], optional
            The icon's weight (i.e., style).

        color : str | tuple, optional
            The color of the icon. May be:
            - a hex code (e.g., `"#03cb98"`)
            - a integer tuple of RGB values, with an optional fourth value for transparency (e.g., `(3, 203, 152, 1)`)
            - any valid color name as determined by the CSS Color Module Level 3 specification
            (https://www.w3.org/TR/css-color-3/#svg-color)

            Hex codes are case-insensitive and the leading `#` is optional.

        size : int | str, optional
            The size of the icon. May be an integer (in pixels) or a CSS size string (e.g., `'1rem'`).

        alt : str, optional
            Alt text for the icon.
        """

phosphor = Phosphor()



import re

import casefy
import reflex as rx
from pydantic import field_serializer
from pydantic_extra_types.color import Color

from RIL._core import Base, Props, validate_props


class BootstrapIconProps(Props):
    color: Color = None
    """
    The color of the icon. May be:
    - a hex code
    - a integer tuple of RGB, RBGA, or HSL
    - any valid color name as determined by the CSS Color Module Level 3 specification 
    (https://www.w3.org/TR/css-color-3/#svg-color)
    
    Hex codes are case-insensitive and the leading `#` is optional.
    """

    size: int | str = None
    """
    The size of the icon. May be an integer (in pixels) or a CSS size string (e.g., `'1rem'`).h
    """

    title: str = None
    """
    An accessible, short-text, description of the icon.
    """

    @field_serializer("color")
    def serialize_color_as_hex(self, color: Color | None):
        return color.as_hex() if color else color


class BootstrapIcon(Base):
    library = "react-bootstrap-icons@^1"

    @classmethod
    @validate_props
    def create(cls, icon: str, props: BootstrapIconProps) -> rx.Component:
        component_model = cls._reproduce(props=props.model_dump())

        component = component_model._create(**props.model_dump())
        component.tag = casefy.pascalcase(icon.casefold())

        if re.match(r"^\d", component.tag):
            component.tag = "Icon" + component.tag

        component.alias = "Bootstrap" + component.tag

        return component


bootstrap = bi = BootstrapIcon.create
